#include "Menu.h"

Menu::Menu (font_t font, Window *parent, bool showTitle, const char *name)
    : curLine (0), title_height (0), Window (parent, name, 4)
{
    this->font = font;
    if (showTitle) {
        title_height = FONT_HEIGHT (font) + 2;
    }
    linesAtOnce = (DISPLAY_ROWS - title_height) / (FONT_HEIGHT (font) + 1);
    /*
    cerr << "With this font, we can fit " << linesAtOnce
         << " lines on the display at once" << endl;

    cerr << "Menu " << name << endl;
    */
}

Menu::~Menu (void)
{
    vector<Line *>::iterator iter = lines.begin();
    for (; *iter; iter++) {
        delete (Line *)*iter;
    }
}

bool
Menu::_onMenuButton (bool pushedAgain)
{
    switchToParent ();
}

bool
Menu::_onKnobLeft (bool)
{
    resetTimer ();
    if (curLine - 1 >= 0) {
        curLine = (curLine - 1) % lines.size();
        draw ();
    }
    return true;
}

bool
Menu::_onKnobRight (bool)
{
    resetTimer ();
    if (curLine + 1 < lines.size()) {
        curLine = (curLine + 1) % lines.size();
        draw ();
    }
    return true;
}

bool
Menu::_onKnobButton (bool)
{
    if (lines[curLine] && lines[curLine]->func) {
        (this->*(lines[curLine]->func))(lines[curLine]);
    }
}

void
Menu::inFocus (void)
{
    resetTimer ();
}

void
Menu::outFocus (void)
{
    clearTimer ();
}

void
Menu::draw (void)
{
    int base, max, i;

    /* Make sure we're in valid land */
    if (curLine >= lines.size())
        curLine = lines.size() - 1;


    if (linesAtOnce < lines.size ()) {
        base = curLine - ((linesAtOnce - 1) / 2);
        if (base < 0) base = 0;
        max = base + linesAtOnce - 1;
        if (max > lines.size () - 1) {
            max = lines.size () - 1;
            base = max - linesAtOnce + 1;
        }
    } else {
        base = 0;
        max = lines.size () - 1;
    }

    /*
    cerr << "base=" << base << "; max=" << max << endl;
    cerr << "curLine=" << curLine << endl;
    */

    clear ();
    for (i = base; lines[i] != NULL && i <= max; i++) {
        Line *line = lines[i];
        drawOneLine (i == curLine, i - base, *line);
    }

    if (title_height) {
        drawTitledWindow (font, name());
    }
    refresh ();
}

void
Menu::drawOneLine (bool reverse, int row, Line &line)
{
    int total_space = ((DISPLAY_ROWS - title_height) -
                                    (linesAtOnce * (FONT_HEIGHT (font) + 1)));
    int this_space = total_space / linesAtOnce;

    if ((this_space * linesAtOnce < total_space) &&
        (total_space - (this_space * linesAtOnce)) >= row) {
        this_space += 1;
    }
    int startY = title_height + row * (FONT_HEIGHT (font) + 1) +
                                                        (this_space * row);

    fillRect (0, startY, DISPLAY_COLS, FONT_HEIGHT (font) + 1, !reverse);
    writeString (title_height ? 2 : 0, startY + 1, font, line.text, reverse);
}

Menu::Line::Line (const char *text, MenuMemberFn func)
{
    this->text = text;
    this->func = func;
}
