#include <sys/fcntl.h>
#include <sys/mman.h>
#include <errno.h>
#include <unistd.h>
#include <sys/ioctl.h>
#include <asm/arch-clps7212/empeg7212.h>
#include "RioDisplay.h"

RioDisplay::RioDisplay (const char *dev, int rows, int cols, int memsize)
    : DisplayDev (rows, cols, memsize), powerState (false)
{
    fd = open (dev, O_RDWR);
    if (fd < 0) {
        perror ("open");
        throw;
    }

    mem = (char *)mmap (0, memsize, PROT_READ|PROT_WRITE,
                        MAP_SHARED, fd, 0);
    if ((int) mem == -1) {
        perror ("mmap");
        close (fd);
        throw;
    }
}

RioDisplay::~RioDisplay (void)
{
    close (fd);
    munmap (mem, memsize());
}

void
RioDisplay::setContrast (int lvl)
{
    if (ioctl (fd, EMPEG_DISPLAY_SETCONTRAST, &lvl) < 0)
        perror ("ioctl - setContrast");
}

int
RioDisplay::getContrast (void)
{
    int lvl = -1;

    if (ioctl (fd, EMPEG_DISPLAY_GETCONTRAST, &lvl) < 0)
        perror ("ioctl - getContrast");

    return lvl;
}

void
RioDisplay::setBacklight (int lvl)
{
    if (ioctl (fd, EMPEG_DISPLAY_SETBACKLIGHT, &lvl) < 0)
        perror ("ioctl - setBacklight");
}

int
RioDisplay::getBacklight (void)
{
    int lvl = -1;

    if (ioctl (fd, EMPEG_DISPLAY_GETBACKLIGHT, &lvl) < 0)
        perror ("ioctl - getBacklight");

    return lvl;
}

void
RioDisplay::setPower (bool on)
{
    powerState = on ? 1 : 0;
    if (ioctl (fd, EMPEG_DISPLAY_POWER, &powerState) < 0)
        perror ("ioctl - setPower");
}

bool
RioDisplay::getPower (void)
{
    return powerState == 1;
}

void
RioDisplay::refresh (void)
{
    if (ioctl (fd, EMPEG_DISPLAY_REFRESH, NULL) < 0)
        perror ("ioctl - refresh");
}
