#include "RootWindow.h"

RootWindow::RootWindow (Window *parent, const char *name)
    : Window (parent, name)
{
    try {
        audio = new RioAudio ("/dev/mixer");
    } catch ( ... ) {
        audio = NULL;
    }

    // Initialize our child windows
    _aboutWindow = new AboutWindow (this);
    _audioMenu = new AudioMenu(audio, this);

    draw ();
    refresh ();
}

void
RootWindow::draw ()
{
    writeString (5, 5, font_helvetica_10, "Root Window");

    drawSpectrum ();
}

#define SPECT_X 2
#define SPECT_Y (DISPLAY_ROWS - SPECT_HEIGHT - 2)
#define SPECT_WIDTH 60
#define SPECT_HEIGHT 35
#define SPECT_A_WIDTH 3

void
RootWindow::drawSpectrum ()
{
    int j = 10;
    int resolution = (SPECT_WIDTH - 3) / (SPECT_A_WIDTH + 1);
    int width = resolution * (SPECT_A_WIDTH + 1) + 3;
    double yscale = resolution / 288; /* log (256) */
    drawRect (SPECT_X, DISPLAY_ROWS - SPECT_HEIGHT - 2, width, SPECT_HEIGHT, 1);

    for (int i = 2; i < width - SPECT_A_WIDTH - 1; i += SPECT_A_WIDTH + 1) {
        fillRect (SPECT_X + i, SPECT_Y + SPECT_HEIGHT - j, SPECT_A_WIDTH, j);
        j++;
    }

}

bool
RootWindow::_onKnobButton (bool pushedAgain)
{
    switchTo (_aboutWindow);
}

bool
RootWindow::_onMenuButton (bool pushedAgain)
{
    switchTo (_audioMenu);
}

bool
RootWindow::_onKnobLeft (bool pushedAgain)
{
    _audioMenu->volumeWindow();
}

bool
RootWindow::_onKnobRight (bool pushedAgain)
{
    _audioMenu->volumeWindow();
}
