#include <qapplication.h>
#include <qpopupmenu.h>
#include <qmessagebox.h>
#include <qmenubar.h>
#include <qstatusbar.h>
#include <qfile.h>
#include <qasyncio.h>
#include <qstring.h>
#include <qtoolbar.h>
#include <qtoolbutton.h>
#include <qnamespace.h>

#include <iostream>
#include <sys/socket.h>
#include <linux/un.h>
#include <unistd.h>
#include <string>
#include "EmuDisplay.h"
#include "ir.h"
#include "XRio.h"

XRio::XRio (int memSock, int ctlSock,int irSock, QWidget *parent,
            const char *name, WFlags f)
    : screen (NULL), _displayServerSocket (NULL),
      _displayCtlServerSocket (NULL), _irServerSocket (NULL),
      _displaySocket (NULL), _displayCtlSocket (NULL), _irSocket (NULL),
      clientConnected (false),
      QMainWindow (parent, name, f)
{

    /* Already connected to application - we're not listening for
     * new connections */
    _displaySocket = new QSocket();
    _displaySocket->setSocket (memSock);

    _displayCtlSocket = new QSocket ();
    _displayCtlSocket->setSocket (ctlSock);

    _irSocket = new QSocket();
    _irSocket->setSocket (irSock);


    connect (_displaySocket, SIGNAL (readyRead()),
             this, SLOT(newDisplayInput()));
    connect (_displaySocket, SIGNAL (connectionClosed()),
             this, SLOT (clientDisc()));
    connect (_displayCtlSocket, SIGNAL (readyRead()),
             this, SLOT(newDisplayCtlInput()));
    connect (_displayCtlSocket, SIGNAL (connectionClosed()),
             this, SLOT (clientDisc()));
    connect (_irSocket, SIGNAL (connectionClosed()),
             this, SLOT (clientDisc()));

    _genericConstructor (parent, name, f);
}

XRio::XRio (const char *memsrc, const char *ctlsrc, const char *irport, 
            QWidget *parent, const char *name, WFlags f)
    : screen (NULL), _displayServerSocket (memsrc),
      _displayCtlServerSocket (ctlsrc), _irServerSocket (irport),
      _displaySocket (NULL), _displayCtlSocket (NULL), _irSocket (NULL),
      QMainWindow (parent, name, f)
{
    /* Connect up the links to the Rio Application */
    connect (&_displayServerSocket, SIGNAL (connectionEstablished(int)),
             this, SLOT (newDisplayConn(int)));
    connect (&_displayCtlServerSocket, SIGNAL (connectionEstablished(int)),
             this, SLOT (newDisplayCtlConn(int)));
    connect (&_irServerSocket, SIGNAL (connectionEstablished(int)),
             this, SLOT (newIRConn(int)));

    _genericConstructor (parent, name, f);
}

void
XRio::_genericConstructor (QWidget *parent, const char *name, WFlags f)
{
    QToolBar *toolbar1, *toolbar2, *toolbar3;
    /* Setup the status bar */
    statusBar()->message ("Initializing virtual Rio Receiver display...");

    /* Setup the menu */
    fileMenu = new QPopupMenu;
    fileMenu->setCheckable (TRUE);
    fileMenu->insertItem ("&Quit", qApp, SLOT(quit()), CTRL+Key_Q);

    menuBar()->insertItem ("&File", fileMenu, ALT+Key_F);

    /* Setup the toolbar */
    toolbar1 = new QToolBar (this, "IR");
    toolbar3 = new QToolBar (this, "IR");

    _powerButton = new QToolButton (toolbar1, "Power");
    _powerButton->setText (_powerButton->name());
    connect (_powerButton, SIGNAL (clicked()), this, SLOT (powerButton()));
    toolbar1->addSeparator();

    _repeatButton = new QToolButton (toolbar1, "Repeat");
    _repeatButton->setText (_repeatButton->name());
    connect (_repeatButton, SIGNAL (clicked()), this, SLOT (repeatButton()));
    toolbar1->addSeparator();

    _randomButton = new QToolButton (toolbar1, "Random");
    _randomButton->setText (_randomButton->name());
    connect (_randomButton, SIGNAL (clicked()), this, SLOT (randomButton()));
    toolbar1->addSeparator();

    _menuButton = new QToolButton (toolbar3, "Menu");
    _menuButton->setText (_menuButton->name());
    connect (_menuButton, SIGNAL (clicked()), this, SLOT (menuButton()));
    toolbar3->addSeparator();

    _knobLeft = new QToolButton (toolbar3, "Knob Left");
    _knobLeft->setText (_knobLeft->name());
    connect (_knobLeft, SIGNAL (clicked()), this, SLOT (knobLeft()));
    toolbar3->addSeparator();

    _knobRight = new QToolButton (toolbar3, "Knob Right");
    _knobRight->setText (_knobRight->name());
    connect (_knobRight, SIGNAL (clicked()), this, SLOT (knobRight()));
    toolbar3->addSeparator();

    _knobPush = new QToolButton (toolbar3, "Knob Push");
    _knobPush->setText (_knobPush->name());
    connect (_knobPush, SIGNAL (clicked()), this, SLOT (knobPush()));

    _playButton = new QToolButton (toolbar1, "Play");
    _playButton->setText (_playButton->name());
    connect (_playButton, SIGNAL (clicked()), this, SLOT (playButton()));
    toolbar1->addSeparator();

    _rewButton = new QToolButton (toolbar1, "Rewind");
    _rewButton->setText (_rewButton->name());
    connect (_rewButton, SIGNAL (clicked()), this, SLOT (rewButton()));
    toolbar1->addSeparator();

    _ffButton = new QToolButton (toolbar1, "FF");
    _ffButton->setText (_ffButton->name());
    connect (_ffButton, SIGNAL (clicked()), this, SLOT (ffButton()));
    toolbar1->addSeparator();

    _stopButton = new QToolButton (toolbar1, "Stop");
    _stopButton->setText (_stopButton->name());
    connect (_stopButton, SIGNAL (clicked()), this, SLOT (stopButton()));

    enableIRButtons (_irSocket != NULL);

    /* Setup the screen itself */
    try {
        screen = new XRioScreen (128, 64, this, "XRioScreen",
                                 WStyle_NormalBorder);
    } catch (QString str) {
        cerr << "Error in XRioScreen creation: " << str << endl;
        throw QString ("XRio creation failed");
    }
    setCentralWidget (screen);
}

XRio::~XRio()
{
    if (screen) delete screen;
}


void
XRio::clientDisc ()
{
    if (!clientConnected)
        return;
    clientConnected = false;

    char buf[4096];
    screen->setBacklightLevel (0);
    memset (buf, 0, sizeof (buf));
    screen->update (buf, sizeof (buf));


    QMessageBox::information (this, "XRio", "Client closed connection");

    if (_irSocket) {
        delete _irSocket;
        _irSocket = NULL;
    }

    if (_displaySocket) {
        delete _displaySocket;
        _displaySocket = NULL;
    }

    if (_displayCtlSocket) {
        delete _displayCtlSocket;
        _displayCtlSocket = NULL;
    }
    enableIRButtons (false);
}

void
XRio::newDisplayCtlInput (void)
{
    int cmd;
    int lvl;
    int rc = _displayCtlSocket->readBlock ((char *)&cmd, sizeof (int)); 
    if (rc != sizeof (int)) {
        cerr << "Only got " << rc << " bytes from control socket" << endl;
    }

    switch (cmd) {
    case EMPEG_DISPLAY_SETCONTRAST:
        if (_displayCtlSocket->readBlock ((char *)&lvl, sizeof (int)) != sizeof (int)) {
            cerr << "Short read from control socket" << endl;
        } else {
            screen->setContrastLevel (lvl);
        }
        break;
    case EMPEG_DISPLAY_GETCONTRAST:
        lvl = screen->getContrastLevel ();
        cerr << "Writing " << lvl << " to socket" << endl;
        _displayCtlSocket->writeBlock ((char *)&lvl, sizeof (int));
        break;
    case EMPEG_DISPLAY_SETBACKLIGHT:
        if (_displayCtlSocket->readBlock ((char *)&lvl, sizeof (int)) != sizeof (int)) {
            cerr << "Short read from control socket" << endl;
        } else {
            screen->setBacklightLevel (lvl);
        }
        break;
    case EMPEG_DISPLAY_GETBACKLIGHT:
        lvl = screen->getBacklightLevel ();
        break;
    case EMPEG_DISPLAY_POWER:
        if (_displayCtlSocket->readBlock ((char *)&lvl, sizeof (int)) != sizeof (int)) {
            cerr << "Short read from control socket" << endl;
        } else {
            screen->setPower (lvl >= 1);
        }
        break;
    default:
        cerr << "Invalid/unknown cmd read: " << cmd << endl;
        break;
    };

}

void
XRio::newDisplayInput (void)
{
    char buf[4096];
    int rv = _displaySocket->readBlock (buf, 4096);

    screen->update (buf, rv);
}

void
XRio::enableIRButtons (bool state)
{
    _powerButton->setEnabled (state);
    _repeatButton->setEnabled (state);
    _randomButton->setEnabled (state);
    _menuButton->setEnabled (state);
    _knobLeft->setEnabled (state);
    _knobRight->setEnabled (state);
    _knobPush->setEnabled (state);
    _playButton->setEnabled (state);
    _rewButton->setEnabled (state);
    _ffButton->setEnabled (state);
    _stopButton->setEnabled (state);
}

QSocket *
XRio::newConn (int sock)
{
    clientConnected = true;
    QSocket *qsock = new QSocket ();
    qsock->setSocket (sock);
    return qsock;
}

void
XRio::newIRConn (int sock)
{
    statusBar()->message ("IR client connected.");
    _irSocket = newConn (sock);
    enableIRButtons (true);
    connect (_irSocket, SIGNAL (connectionClosed()),
             this, SLOT (clientDisc()));
}

void
XRio::newDisplayConn (int sock)
{
    statusBar()->message ("Display client connected.");
    _displaySocket = newConn (sock);
    connect (_displaySocket, SIGNAL (readyRead()),
             this, SLOT(newDisplayInput()));
    connect (_displaySocket, SIGNAL (connectionClosed()),
             this, SLOT (clientDisc()));
    screen->setBacklightLevel (1);
    screen->setContrastLevel (DEFAULT_CONTRAST_VALUE);
}

void
XRio::newDisplayCtlConn (int sock)
{
    statusBar()->message ("Display control client connected.");
    _displayCtlSocket = newConn (sock);
    connect (_displayCtlSocket, SIGNAL (readyRead()),
             this, SLOT(newDisplayCtlInput()));
    connect (_displayCtlSocket, SIGNAL (connectionClosed()),
             this, SLOT (clientDisc()));
}

void
XRio::irOut (uint32_t code)
{
    write (_irSocket->socket(), &code, sizeof (code));
}


void
XRio::powerButton ()
{
    statusBar()->message ("Power button pushed");
    irOut (PANEL_POWER_DN);
    irOut (PANEL_POWER_UP);
}

void
XRio::repeatButton ()
{
    statusBar()->message ("Repeat button pushed");
    irOut (PANEL_REPEAT_DN);
    irOut (PANEL_REPEAT_UP);
}

void
XRio::randomButton ()
{
    statusBar()->message ("Random button pushed");
    irOut (PANEL_RANDOM_DN);
    irOut (PANEL_RANDOM_UP);
}

void
XRio::menuButton ()
{
    statusBar()->message ("Menu button pushed");
    irOut (PANEL_MENU_DN);
    irOut (PANEL_MENU_UP);
}

void
XRio::knobLeft ()
{
    statusBar()->message ("Knob turned left");
    irOut (PANEL_KNOB_LEFT);
}

void
XRio::knobRight ()
{
    statusBar()->message ("Knob turned right");
    irOut (PANEL_KNOB_RIGHT);
}

void
XRio::knobPush ()
{
    statusBar()->message ("Knob button pushed");
    irOut (PANEL_KNOB_BTN_DN);
    irOut (PANEL_KNOB_BTN_UP);
}

void
XRio::playButton ()
{
    statusBar()->message ("Play button pushed");
    irOut (PANEL_PLAY_DN);
    irOut (PANEL_PLAY_UP);
}

void
XRio::rewButton ()
{
    statusBar()->message ("Rew button pushed");
    irOut (PANEL_REW_DN);
    irOut (PANEL_REW_UP);
}

void
XRio::ffButton ()
{
    statusBar()->message ("FF button pushed");
    irOut (PANEL_FF_DN);
    irOut (PANEL_FF_UP);
}

void
XRio::stopButton ()
{
    statusBar()->message ("Stop button pushed");
    irOut (PANEL_STOP_DN);
    irOut (PANEL_STOP_UP);
}
