#include <qmainwindow.h>
#include <qtoolbutton.h>
#include <qasyncio.h>
#include <qsocket.h>
#include "XRioScreen.h"

class XRio : public QMainWindow
{
    Q_OBJECT
public:
    XRio (int memSock, int ctlSock, int irSock, QWidget *parent = 0,
          const char *name = 0, WFlags f = WType_TopLevel);
    XRio (const char *memsrc, const char *ctlsrc, const char *irfile,
          QWidget *parent = 0, const char *name = 0, WFlags f = WType_TopLevel);
    ~XRio();

public slots:
    void newDisplayInput (void);
    void newDisplayCtlInput (void);
    void clientDisc (void);
    void newDisplayConn (int sock);
    void newDisplayCtlConn (int sock);
    void newIRConn (int sock);
    QSocket *newConn (int sock);
    void powerButton ();
    void repeatButton ();
    void randomButton ();
    void menuButton ();
    void knobLeft ();
    void knobRight ();
    void knobPush ();
    void playButton ();
    void rewButton ();
    void ffButton ();
    void stopButton ();

private:
    void _genericConstructor (QWidget *parent, const char *name, WFlags f);
    void irOut (uint32_t val);
    void enableIRButtons (bool state);

private:
    XRioScreen *screen;

private: // menu
    QPopupMenu *fileMenu;

private: // toolbar
    QToolBar *toolbar;
    QToolButton *_powerButton;
    QToolButton *_repeatButton;
    QToolButton *_randomButton;
    QToolButton *_menuButton;
    QToolButton *_knobLeft;
    QToolButton *_knobRight;
    QToolButton *_knobPush;
    QToolButton *_playButton;
    QToolButton *_rewButton;
    QToolButton *_ffButton;
    QToolButton *_stopButton;

private: // communications
    UnixServerSocket _irServerSocket;
    QSocket *_irSocket;
    UnixServerSocket _displayServerSocket;
    QSocket *_displaySocket;
    UnixServerSocket _displayCtlServerSocket;
    QSocket *_displayCtlSocket;
    bool clientConnected;
};
