
//
// Play a raw audio file consisting of 16-bit stereo samples,
// little endian format from stdin.
// 
// Largely lifted from Rob Leslie's Empeg demo code and 
// adapted to the particulars of the Rio Receiver.
//
// Jeff Mock
// jeff@mock.com
// 2/27/1
//

#include <stdio.h>
#include <fcntl.h>
#include <sys/soundcard.h>
#include <asm/arch-clps7212/empeg7212.h>

#define AUDIO_DEVICE    "/dev/audio"
#define MIXER_DEVICE    "/dev/mixer"
#define AUDIO_FILLSZ    4608

static  char buffer[AUDIO_FILLSZ];
static  int  fdaudio, fdmixer;

//
// Must output buffers of 4608 bytes or audio driver barfs.  
// Sample rate is always 44.1k, format always 16-bit stereo, alternating 
// left/right samples.
//
static void 
output(void *buf)
{
    int wrote = write(fdaudio, buf, AUDIO_FILLSZ);
    if (wrote != AUDIO_FILLSZ) {
        printf("Audio write failed: %d.\n", wrote);
        exit(1);
    }
}

static void
volume(int vol)
{
    vol = (vol & 0xff) | ((vol << 8) & 0xff00);
    if (ioctl(fdmixer, MIXER_WRITE(SOUND_MIXER_VOLUME), &vol) == -1) {
        perror("ioctl(mixer volume)");
        exit(1);
    }
}

static void
balance(int bal)
{
    if (ioctl(fdmixer, EMPEG_MIXER_WRITE_BALANCE, &bal) == -1) {
        perror("ioctl(mixer balance)");
        exit(1);
    }
}

static void
soft_mute(int m)
{
    if (ioctl(fdmixer, EMPEG_MIXER_SET_SAM, &m) == -1) {
        perror("ioctl(SAM)");
        exit(1);
    }
}

static void
hard_mute(int m)
{
    if (ioctl(fdmixer, EMPEG_MIXER_SET_HAM, &m) == -1) {
        perror("ioctl(HAM)");
        exit(1);
    }
}

#if 0
int 
main(int ac, char *av)
{
    int     bc=0;
    int     vol, bal;
    unsigned int len;

    fdaudio = open(AUDIO_DEVICE, O_WRONLY);
    if (fdaudio < 0) {
        printf("Cannot open audio device\n");
        exit(1);
    }

    fdmixer = open(MIXER_DEVICE, O_WRONLY);
    if (fdmixer < 0) {
        printf("Cannot open mixer\n");
        exit(1);
    }

    volume(100);
    balance(0);
    hard_mute(0);
    soft_mute(0);
 
    while (fread(buffer, 1, AUDIO_FILLSZ, stdin) == AUDIO_FILLSZ) 
        output(buffer);

    printf("Done.\n");
    soft_mute(1);
    close(fdmixer);
    close(fdaudio);
}
#endif
